import { ASTPath, Collection, JSCodeshift, JSXAttribute } from 'jscodeshift';
import { ButtonVariant, ButtonTone, OldVariant } from '../../../../src/types';

type VariantMapperType = {
  [key in OldVariant]: {
    variant: ButtonVariant;
    tone?: ButtonTone;
  };
};

const oldThemeToNewVariantAndTone: VariantMapperType = {
  default: {
    variant: 'default',
  },
  primary: {
    variant: 'emphasized',
    tone: 'default',
  },
  critical: {
    variant: 'emphasized',
    tone: 'critical',
  },
  success: {
    variant: 'emphasized',
    tone: 'success',
  },
  tertiary: {
    variant: 'plain',
  },
};

export default function updateTheme(j: JSCodeshift, source: Collection) {
  source.findJSXElements('Button').forEach((element) => {
    const attributes = (element.value.openingElement.attributes ??
      []) as unknown as ASTPath<JSXAttribute>[];
    const themeAttribute = attributes.find(
      (attribute) => attribute?.name?.name === 'theme',
    );

    if (themeAttribute == null) {
      return;
    }
    const valueType = themeAttribute.value.type as string;
    themeAttribute.name = 'variant';
    const isLiteral = valueType === 'Literal' || valueType === 'StringLiteral';

    if (!isLiteral) {
      return;
    }

    const value = themeAttribute.value.value as unknown as ButtonVariant;

    const newProps = oldThemeToNewVariantAndTone[value as OldVariant];

    if (newProps == null) {
      return null;
    }

    themeAttribute.value.value = newProps.variant as any;

    if (newProps.tone) {
      // eslint-disable-next-line no-param-reassign
      element.node.openingElement.attributes = [
        ...(attributes as any),
        j.jsxAttribute(j.jsxIdentifier('tone'), j.stringLiteral(newProps.tone)),
      ];
    }
  });
}
